'******************************************************************************
'*                                                                            *
'*                   Arbitrary Waveform Data Generator Example                *
'*                                                                            *
'*    Eric Cox                                                                *
'*    (650) 938-0240                                                          *
'*    Fax: (650) 938-0241                                                     *
'*                                                                            *
'*                                                                            *
'******************************************************************************

' Program Description

' This program generates a sinewave and downloads the data points to the
'  unit for generation with the Arbitrary Waveform Generator.

'  It can generate the data in a variety of different number formats.
'  Each format has its own subroutine below.


' ------- Open the serial port for communication with the unit ------------

' Ask the user which serial port is attached to the unit

   INPUT "Is the unit on Com port 1 or 2?  (1/2)  >"; a$

' Open Com port, Disable CD,CTS,DCD, give 5 sec max to open,
'   Set Rx buffer to 2K, Disable RTS, set Tx buff to 1K, Random = i/o,
'   Set random access buffer size to 2K

   IF a$ = "1" THEN
      OPEN "COM1:9600,n,8,1,CD0,CS0,DS0,OP5000,RB2048,RS,TB1024" FOR RANDOM AS #1 LEN = 2048
      GOTO EndPortSel
   END IF

   IF a$ = "2" THEN
      OPEN "COM2:9600,n,8,1,CD0,CS0,DS0,OP5000,RB2048,RS,TB1024" FOR RANDOM AS #1 LEN = 2048
      GOTO EndPortSel
   END IF

   GOTO ArbExit         ' Abort program if neither Com1 nor Com2 was selected

EndPortSel:


' ---- Command the unit to enter Arbitrary Waveform Generation Mode -------
' Note: The unit does not need to be in Arbitrary Waveform generation mode
'       to accept data points over the serial port. We switch to Arbitrary
'       waveform mode here as an example of remotely controlling the operation
'       of the unit, and to have the waveform generated after the download.
'       For more information on Remote Control, See the Users Manual

' Put the unit in arbitrary waveform mode, set parameters

   PRINT #1, "ML F11 F240Z F35Y"  ' ML = Set Arbitrary Mode
                                  ' F11 = Set Continuous (not Triggered) mode
                                  ' F240Z = Set sampling clock to 40 MHz
                                  ' F35Y = Set output level to 5V p-p



' ------------------------------- Main Loop --------------------------------
' This loop calculates each point of the arbitrary waveform.
' It then calls one of several subroutines (depending on what format you
'   would like the data in) to send the data to the unit 

   
    NUMPOINTS = 80              ' Total number of points generated
   
    Phase# = 0                  ' Reset sinewave phase accumulator to 0

    PhaseInc# = 2 * 3.1415926# / NUMPOINTS    ' Phase goes 0 to 2*Pi

' This loop generates the points and sends them to the unit 

    FOR PointNumber = 1 TO NUMPOINTS

' Note: Here we call a function to generate a new point value. Basic returns
'       a value from SIN() which ranges from -1.0 to +1.0.  You may insert
'       your own function here, but be sure that the returned value ranges
'       from -1.0 to +1.0 since the subroutines below expect "PointVal"
'       to range from -1.0 to +1.0.


    PointVal = SIN(Phase#)
   
' This function generates a positive ramp from -1.0 to +1.0
'    PointVal = -1 + 2 * PointNumber / NUMPOINTS
' This function generates a falling exponential from +1.0 to 0
'    PointVal = EXP(-3 * PointNumber / NUMPOINTS)


' Pick one of the following subroutines depending on desired data format:
'    GOSUB SendFloat       ' Send Floating Point, ex: 3.67e-2, -.586, .012e1,...
    GOSUB SendHex        ' Send Hexadecimal, ex: fed7,124,63c8...
'    GOSUB SendInteger    ' Send Integer, ex: -2047,185,2047...
'    GOSUB SendBinary     ' Send Binary, ie <hi byte>,<low byte>,...

    Phase# = Phase# + PhaseInc#        ' Advance the phase of our sinusoid

    NEXT PointNumber                   ' Continue loop and generate more points


' All points have now been generated and sent to the unit.
' When the unit hasn't received any chars over the serial port for a
' 1 second timeout, it assumes that all points have been sent and starts
' generating the waveform.

ArbExit:
    CLOSE : SYSTEM              ' Close the comm port and exit the program.



'******************************************************************************
'                               Subroutines
'
'  These subroutines take a value in PointVal (which ranges from -1.0 to +1.0)
'   and converts it to various data formats, (i.e. Floating Point, Binary,
'   Ascii Hex, and Integer) and then sends that value to the unit.
'
'  Before the first data point is sent, a Header consisting of two characters
'   is sent to the unit.  The first character of this header is a "W", which
'   tells the unit to expect arbitrary waveform data points. The second
'   character of the header tells the unit what data format to expect,
'
'   "F" = Floating Point
'   "I" = Integer
'   "H" = Ascii Hexadecimal
'   "B" = Binary
'
' Each subroutine gives a short example of each format.
'
'******************************************************************************



'--------------------- Send Floating Point data to unit ---------------------
' This is the easiest format to use with Basic since no format conversion is
'  necessary. Basic will output floating point numbers by default.
'
' Rules for floating point format:
'
' 1. The unit expects all floating point values to be between -1.0 and +1.0.
'    If a number falls outside that range, the number is set to -1.0 or +1.0.
'    These correspond to the peak values of the waveform.
'    If the output voltage level were set for 5V p-p, for example, then +1.0
'    corresponds to +2.5V and -1.0 corresponds to -2.5V (across a 50 ohm load).
'    For maximum signal fidelity, you should design your waveform so that the
'    peak values "just fit" into the range of -1.0 to +1.0. You will then be
'    using the full output range of the DAC.
'
' 2. To separate each number, you may use commas, tabs, spaces, carriage
'    returns and/or linefeeds, semicolons, colons or basically any character
'    that is not among the following: 
'     0   1   2   3   4   5   6   7   8   9   .   -   +   e   E
'
' 3. You do not have to include an exponent (i.e. e-5). If no exponent is
'    given, the unit assumes that the exponent is 0.
'
' 4. Whitespace characters are not allowed between the mantissa and exponent.
'
' 5. Use of the "+" character for positive values is optional.
'
' 6. To set the Pulse Output high during a data point, put a "p" or "P"
'    BEFORE that data point. The "P" character may have whitespace around
'    it if desired.  The Pulse Output is set low for all data points that
'    do not have a "p" or "P" before them.
'
' Examples of floating point numbers, the Pulse Output is set on point #4:
'
'   0, .584737, 3457e-4, p  .0004857e+3  -.000485  -1.0e-0
'------------------------------------------------------------------------------

SendFloat:

   IF PointNumber = 1 THEN PRINT #1, "WF"  ' Put header info before 1st data point

   IF PointNumber = 2 THEN PRINT #1, "p"   ' Set Pulse output bit for second point

   PRINT #1, PointVal                      ' Send Floating point number to unit

   RETURN




'------------------------- Send Integer data to unit ----------------------
' This format sends a base 10 number which is in the range -2047 to +2047
'
'  The integer numbers represent -1.0 to +1.0 as follows:
'
'              -2047 ---- -1024 ---  0 ---- 1024 ---- 2047
'               -1.0        -.5     0.0     +.5       +1.0
'
' Rules for Integer format:
'
' 1. The unit expects all Integer values to be between -2047 and +2047.
'    If a number falls outside that range, the number is set to -2047 or +2047.
'    These correspond to the peak values of the waveform. If the output voltage
'    level were set for 5V p-p, for example, then +2047 corresponds to +2.5V
'    and -2047 corresponds to -2.5V (across a 50 ohm load).  For maximum signal
'    fidelity, you should design your waveform so that the peak values "just
'    fit" into the range of +2047 to -2047. You will then be using the full
'    output range of the DAC.
'
' 2. To separate each number, you may use commas, tabs, spaces, carriage
'    returns and/or linefeeds, semicolons, colons or basically any character
'    that is not among the following: 
'             0   1   2   3   4   5   6   7   8   9   .   -   +
'
' 3. Use of the "+" character for positive values is optional.
'
' 4. Leading "0"'s are ignored.
'
' 5. To set the Pulse Output high during a data point, put a "p" or "P"
'    BEFORE that data point. The "P" character may have whitespace around it
'    if desired.  The Pulse Output is set low for all data points that do not
'    have a "p" or "P" before them.
'
' Examples of integer numbers, the pulse output is set on point #3:
'
'   0, 123 p -411  -320 +12,  +2017,  -2047
'------------------------------------------------------------------------------


'---------------------- Send data point as Integer -------------------------
SendInteger:

   IF PointNumber = 1 THEN PRINT #1, "WI"  ' Put header before 1st data point

   IF PointNumber = 2 THEN PRINT #1, "p"  ' Set Pulse output for second point

' Convert the value in PointVal to an integer that ranges from -32767 to +32767

   a = INT(PointVal * 2047)    ' Convert -1.0, +1.0 val to a -2047, +2047 val

   PRINT #1, a                  ' Send integer value to unit

   RETURN






'------------------------- Send Ascii Hex data to unit ----------------------
' This format sends the same two's complement data that Binary format uses,
'  except that each nibble (4 bits) of the hex value is represented by
'  an ascii character.
'
'  Each point is a 16 bit word which is sent to the unit with 4 characters.
'  Each character represents 4 bits of the 16 bit value:
'
'   0000 to 1001 - Represented by an ascii "0" to "9"
'   1010         - Represented by an ascii "a" or "A"
'   1011         - Represented by an ascii "b" or "B"
'   1100         - Represented by an ascii "c" or "C"
'   1101         - Represented by an ascii "d" or "D"
'   1110         - Represented by an ascii "e" or "E"
'   1111         - Represented by an ascii "f" or "F"
'
' For example, the 16 bit value  1101 0011 0101 1111 is sent as "d35f"
'
'
'  The 16 bit value is in "two's complement" format, which represents a number
'   from -1.0 to +1.0 as follows:
'
'               8000 ---- E000 ---- FFFF,0 ---- 4000 ---- 7FFF
'               -1.0      -.5        0.0         +.5      +1.0
'
' Although a 16 bit value is sent to the unit, the Arbitrary Waveform system
'  uses a 12 bit D/A converter, so not all 16 bits are used. Only the uppermost
'  12 bits are used to form the arbitrary waveform point.
'
'
' Rules for Hexadecimal format:
'
' 1. Each Hexadecimal value may have 1 to 4 characters. Negative values require
'   4 characters since the most significant bit of the value (bit 15)
'   must be a 1 for negative values. If there are fewer than 4 characters the
'   value is interpreted as positive. Characters are not case sensitive.
'
' 2. To separate each number, you may use commas, tabs, spaces, carriage returns
'   and/or linefeeds, semicolons, colons or basically any character that is not
'   among the following:
'       0  1  2  3  4  5  6  7  8  9  a  A  b  B  c  C  d  D  e  E  f  F
'
' 3. The most significant nibble character is the first that is sent to the
'    unit. For example, to send the value 1234, the 1 is sent first, then
'    the 2, etc.
'
' 4. The pulse out output is controlled by bit #3 in the low byte. If this bit
'    is set to 1, the pulse out is set high. If this bit is set to 0, the pulse
'    output is set low.
'
' Examples of ascii hex numbers, the pulse output is set on point #3:
'
'   0, 4000,  fed8  4570   8000  fff0  E6D0, 10   FF,C06
'------------------------------------------------------------------------------

SendHex:

   IF PointNumber = 1 THEN PRINT #1, "WH"  ' Put header before 1st data point


' Convert the value in PointVal to two's complement hex
    j = 32767

    IF PointVal < 0 THEN PointVal = PointVal + 2: j = 32768!
    a = INT(PointVal * j)             ' Get a number 0 to 65535

    a = INT(a / 16)      ' Mask off lower 4 bits ($FFF0)
    a = a * 16

   IF PointNumber = 2 THEN a = a OR &H8  ' Set Pulse output bit for second point

   a$ = HEX$(a)                        ' Convert the integer to hexadecimal

' For negative values, the HEX$ fctn will return a 32 bit value, ie $FFFF FFD8,
' so chop off all but the last 4 chars

  IF a < 0 THEN a$ = RIGHT$(a$, 4)

   PRINT #1, a$                 ' Send hex point to unit

   RETURN




'------------------------- Send Binary data to unit ------------------------
' This is the fastest way to send points to the unit since it transfers
'  the data point with only 2 characters. It is also the least forgiving
'  as far as getting the data sent correctly.
'
'  Each data point is a 16 bit word which is sent to the unit in two bytes.
'  The high byte is sent first, followed by the low byte.
'  The 16 bit value is in "two's complement" format, which represents a number
'   from -1.0 to +1.0 as follows:
'
'                 8000 ---- E000 ---- FFFF,0 ---- 4000 ---- 7FFF
'                 -1.0      -.5        0.0         +.5      +1.0
'
' Although a 16 bit value is sent to the unit, the Arbitrary Waveform system
'  uses a 12 bit D/A converter, so not all 16 bits are used. Only the uppermost
'  12 bits are used to form the arbitrary waveform point.
'
'
' Rules for Binary format:
'
' 1. IMMEDIATELY after the "B" character in the header the unit expects the
'    first high byte of data point 1. No whitespace is allowed after the "B"
'    in Binary mode.
'
' 2. The high byte of each data point is sent first, followed by the low byte.
'
' 3. The pulse out output is controlled by bit #3 in the low byte. If this bit
'    is set to 1, the pulse out is set high. If this bit is set to 0, the pulse
'    output is set low.
'------------------------------------------------------------------------------

SendBinary:

   IF PointNumber = 1 THEN PRINT #1, "WB";  ' Put header info before 1st data
                                           ' point. Must not have CR or LF after

' Convert the value in PointVal to a two's complement hex number
    j = 32767
    IF PointVal < 0 THEN PointVal = PointVal + 2: j = 32768!
   
    a = INT(PointVal * j)             ' Convert -1.0,+1.0 to 0,65535

    a = INT(a / 16)                   ' Only the uppermost 12 bits are used, so
    a = a * 16                        ' Mask off the lower 4 bits (AND fff0)

   IF PointNumber = 2 THEN a = a OR &H8  ' Set Pulse output bit for second point

' If you let Basic send a 16 bit variable to a port, it will send the low byte
'  first and then the high byte.
' The unit expects the high byte first so we must reverse the two so we
'  get the proper byte order (i.e. high byte then low byte)

   high = INT(a / 256)          ' Get the High byte of the 16 bit value
   low = a MOD 256              ' Get the Low byte of the 16 bit value
   a = low * 256 + high         ' Switch hi and low bytes

   IF a > 32767 THEN a = a - 32768  ' Make it signed for conversion to int.
   
   b% = a                      ' Convert to a 16 bit variable type
   PUT #1, , b%                ' Write low byte, then hi byte to file

   RETURN


