
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------
'                 Function Generator Remote Control Example
'                          Application Program A
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------

' This program is written in Microsoft QuickBasic

'     This program does the following:
'        
'     1. Clear the screen
'     2. Open the serial communications port for communicating with the unit
'     3. Make sure there is a unit connected to the serial port.  If we can't 
'         find one, wait until it is connected up. 
'     4. Command the unit to enter Burst mode.
'     5. Set up the burst frequency, on time and off time.  On time is the burst
'         duration, off time is the burst delay.
'     6. Present a menu to the user asking which remote control command to 
'         issue (i.e. power on/off, change channel, etc.)  
'     7. When a command is selected, command the unit to generate the same 
'         number of pulses associated with the selected command.


DECLARE SUB Delay (X%)     ' Declare functions used below.
                           '  (The Delay function waits x mS then returns)
DECLARE SUB Pause ()       '  (The Pause fctn waits for a keypress then returns)

DIM Array(5)               ' General purpose array

'------------------------- User Entered Values ---------------------------------
    
' By observing the signal from a remote control with an oscilloscope, the 
'  following parameters were recorded.  To adapt this program to another 
'  remote control model, change the values below:

CarrierFreq$ = "32768"          ' Type in the infrared carrier freq here.
                                '  this value is in Hz
OnTime$ = "30"                  ' Type in the burst on time here.
                                '  this value is in mS
OffTime$ = "10"                 ' Type in the burst off time here.
                                '  this value is in mS

' These variables hold the number of pulses that form each command:

PowerToggle = 20                '  Turn the TV on and off (twenty pulses)
ChannelUp = 9                   '  Go to next channel
ChannelDown = 12                '  Go to previous channel
VolumeUp = 3                    '  Increase volume one notch
VolumeDown = 5                  '  Decrease volume one notch

'-------------------------------------------------------------------------------

   CLS                          ' Clear the screen

' This array is used as a data table which, given a command menu number as an
'  index, returns the number of pulses in a given infrared pulse stream.

   Array(1) = PowerToggle
   Array(2) = ChannelUp
   Array(3) = ChannelDown
   Array(4) = VolumeUp
   Array(5) = VolumeDown

' Open serial port 1 for communication with the unit with these parameters:
'   Ignore the CD,CTS,DCD flow control lines
'   Set a 5 sec maximum timeout for basic to open the port
'   Set the receive buffer to 2K bytes
'   Disable the RTS flow control line
'   Set the transmit buffer to 1K bytes
'   File access type is Random
'   Set random access buffer size to 2K

   OPEN "COM1:9600,n,8,1,CD0,CS0,DS0,OP5000,RB2048,RS,TB1024" FOR RANDOM AS #1 LEN = 2048

'----------- Make sure a unit is attached to the serial port --------------

' If we send the unit a control E character, it'll respond with a control C.
'  We use this feature to see if a unit is attached.  If a unit cannot be 
'  found, wait until one is attached.
' User can abort the program at this point by pressing any key.

' When power is applied to the unit, it prints a menu to the serial port.
' We wish to disregard these characters, so flush them.

   GOSUB FlushBuff1             ' Flush all received chars from buffer

' Tell the user to connect up a function generator

   PRINT "Connect the function generator to the serial port and power the unit"

ConnectLoop:
   PRINT #1, CHR$(5);           ' Print a control E to unit (ascii char 5)

' Wait for the unit to give us some characters. When they arrive, put
'  then into the string a$

   GOSUB GetResponse1           ' Wait for and get response to a$

' Allow the user to exit the program by pressing a key.

   IF INKEY$ <> "" THEN SYSTEM  ' Exit on first terminal keypress

' Did we get a control C (ascii char 3) from the unit?

   IF INSTR(a$, CHR$(3)) THEN GOTO GotSg100

   GOTO ConnectLoop
GotSg100:

'----------- Switch the generator to Burst mode --------------
' Send the following command string to the unit:
' A M6 F2 xxX F1 xxX F3 xxX F0
' These characters have the following meaning:

'   A      - Reset the unit to Sinewave mode (ensures the state of generator)
'   M6     - Go to Burst Mode
'   F3     - Move cursor to field 3 (the burst frequency field)
'  xxX     - Enter a frequency of xx Hz (infrared carrier freq)
'   F2     - Move cursor to field 2 (the burst duration field)
'  xxX     - Enter a duration of xx mS (burst on time)
'   F4     - Move cursor to field 4 (the burst delay field)
'  xxX     - Enter a duration of xx mS (burst off time)
'   F0     - Move cursor to field 0 (turns cursor off)

   PRINT #1, "A M6 F3";
   PRINT #1, CarrierFreq$; "X";         ' Send measured carrier freq
   PRINT #1, "F2 ";
   PRINT #1, OnTime$; "X";              ' Send burst on time
   PRINT #1, "F4 ";
   PRINT #1, OffTime$; "X";             ' Send burst off time
   PRINT #1, "F0";

'  The unit will now trigger a single burst for each "t" char we sent it.   
'  After the unit executes each "t" command, it will return a prompt
'  character  (">"). We wish to wait until after all prompts have come
'  in, since that's  when the unit has finished processing the last "t".
'  The GetResponse1 subroutine has a timeout feature which will accomplish
'  this purpose.

   GOSUB GetResponse1           ' Wait for and get response to a$

GetCommand:

     CLS

     PRINT : PRINT
     PRINT "     1. Power on/off"
     PRINT "     2. Channel Up"
     PRINT "     3. Channel Down"
     PRINT "     4. Volume Up"
     PRINT "     5. Volume Down"
     PRINT
     PRINT "    ESC To exit program"
     PRINT
     PRINT "  Please select a command >"

CommandPoll:
     a$ = INKEY$
     IF a$ = "" THEN GOTO CommandPoll

     IF a$ = CHR$(27) THEN SYSTEM       ' Exit the program on ESC keypress

     IF a$ < "1" OR a$ > "5" THEN GOTO GetCommand       ' Is input valid?

     PRINT : PRINT
     PRINT "Command sent."              ' Tell user command will be sent.

     a = VAL(a$)                ' Convert selection to a number, 1 to 5

     NumPulses = Array(a)       ' Look up no. of pulses for this command

' The variable NumPulses now has the number of infrared pulses to form this 
'  remote control command. Download this many "t" characters to the unit
'  which causes it to trigger that many pulses in Burst mode.

     a$ = STRING$(NumPulses, "t")       ' Form a string with NumPulses number
                                        '  of "t" chars in it. I.e., if
                                        '  Numpulses=3, then a$="ttt"

     PRINT #1, a$;              ' Dump the "t"'s to the unit

' The unit will now trigger a burst for each "t" char we sent it.  After 
'  the unit executes each "t" command, it will return a prompt character
'  (">"). We wish to wait until after all prompts have come in, since that's 
'  when the unit has finished processing the last "t".   The GetResponse1 
'  subroutine has a timeout feature which will accomplish this purpose.

   GOSUB GetResponse1           ' Wait for and get response to a$

   GOTO GetCommand              ' Go get another remote control command


'------------------------------------------------------------------------------
'------------------------------------------------------------------------------
'                              SUBROUTINES
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------


'------------------------------------------------------------------------------
'                     Get ascii response from the unit 
' Wait for an incoming char, then collect incoming stuff until we get a
'   800 mS Rx char timeout
' Returns with a$ = received stuff
'------------------------------------------------------------------------------
GetResponse1:

   IF LOC(1) = 0 THEN GOTO GetResponse1     ' Wait for incoming char

   a$ = ""

GetRespLoop1:

   a$ = a$ + INPUT$(LOC(1), #1)             ' Get all waiting chars in Rx buffer

' Now wait for a timeout time to make sure no new stuff is coming in.
   CALL Delay(.8)                           ' Do a 800 mS delay

   IF LOC(1) <> 0 THEN GOTO GetRespLoop1    ' If something came in during
                                            '  our timeout, grab it
   RETURN

'------------------------------------------------------------------------------
'                 Wait for a command prompt 
' Returns A$ = all stuff received before the command prompt.
'------------------------------------------------------------------------------
PromptWait:

' If we have no chars in Rx buffer, wait until we have 1 or more

   a$ = ""

PromptWaitLoop:
   IF LOC(1) = 0 THEN GOTO PromptWaitLoop

   a$ = a$ + INPUT$(LOC(1), #1)    ' Get all waiting chars in Rx buffer

   IF INSTR(a$, ">") THEN GOTO GotPrompt

   GOTO PromptWaitLoop

' Here we got a prompt. Exit
GotPrompt:

   RETURN


'------------------------------------------------------------------------------
'                 Flush all Rx chars from Port 1 buffer
'------------------------------------------------------------------------------
FlushBuff1:

   IF LOC(1) = 0 THEN GOTO DoTimeout1

   a$ = INPUT$(LOC(1), #1)      ' Get all waiting chars in Rx buffer

' Now wait for a timeout time to make sure no new stuff is coming in.
DoTimeout1:
   CALL Delay(.8)               ' Do a 800 mS delay

   IF LOC(1) <> 0 THEN GOTO FlushBuff1          ' If something came in during
                                                '  our timeout, flush it
   RETURN

'------------------------------------------------------------------------------
'                             Wait for x mS
' Function to delay x number of mS.
'------------------------------------------------------------------------------
  SUB Delay (X%) STATIC

   start! = TIMER                       ' Returns no. of secs since midnight.
                                        ' Example: 65445.15

DelayLoop:                              ' Do a x mS delay
   IF TIMER < start! + X% / 1000 THEN GOTO DelayLoop

  END SUB

'------------------------------------------------------------------------------
'                          Wait for a keypress
'------------------------------------------------------------------------------
  SUB Pause STATIC

   DO
   LOOP UNTIL INKEY$ <> ""       ' Wait for keypress to continue

   END SUB

